UserWsClient

Struct UserWsClient 

pub struct UserWsClient { /* private fields */ }
WebSocket client for streaming authenticated user events

This client connects to the Polymarket CLOB user WebSocket endpoint and streams real-time updates about the user’s trades and orders.

Connection Management

The Polymarket WebSocket server will disconnect idle connections after 1-2 minutes. For production use, it’s recommended to use ReconnectingStream to automatically handle disconnections and reconnect with exponential backoff.

Example with Auto-Reconnect

use polymarket_rs::websocket::{UserWsClient, ReconnectConfig, ReconnectingStream};
use polymarket_rs::types::ApiCreds;
use futures_util::StreamExt;
use std::time::Duration;

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let creds = ApiCreds::new(
        "api_key".to_string(),
        "api_secret".to_string(),
        "api_passphrase".to_string(),
    );

    let client = UserWsClient::new();

    let config = ReconnectConfig {
        initial_delay: Duration::from_secs(1),
        max_delay: Duration::from_secs(30),
        multiplier: 2.0,
        max_attempts: None,
    };

    let creds_clone = creds.clone();
    let mut stream = ReconnectingStream::new(config, move || {
        let client = client.clone();
        let creds = creds_clone.clone();
        async move { client.subscribe_with_creds(&creds).await }
    });

    while let Some(event) = stream.next().await {
        match event {
            Ok(evt) => println!("Event: {:?}", evt),
            Err(_) => continue, // Will auto-reconnect
        }
    }
    Ok(())
}

Implementations

Source

impl UserWsClient

pub fn new() -> Self

Create a new user WebSocket client with the default endpoint

pub fn with_url(ws_url: impl Into<String>) -> Self

Create a new user WebSocket client with a custom endpoint

pub async fn subscribe_with_creds( &self, creds: &ApiCreds, ) -> Result<Pin<Box<dyn Stream<Item = Result<UserWsEvent>> + Send>>>

Subscribe to user events with API credentials

Returns a stream of UserWsEvent items. The stream will yield events as they are received from the WebSocket connection.

Arguments
  • creds - API credentials (api_key, secret, passphrase)
Events

The stream will yield two types of events:

Errors

Returns an error if:

  • The WebSocket connection fails
  • The authentication message cannot be sent
  • Authentication fails (server will close the connection)
Example
let creds = ApiCreds::new(
    "your_api_key".to_string(),
    "your_api_secret".to_string(),
    "your_api_passphrase".to_string(),
);

let client = UserWsClient::new();
let mut stream = client.subscribe_with_creds(&creds).await?;

while let Some(event) = stream.next().await {
    println!("Event: {:?}", event?);
}
pub async fn subscribe( &self, api_key: String, api_secret: String, api_passphrase: String, ) -> Result<Pin<Box<dyn Stream<Item = Result<UserWsEvent>> + Send>>>

Subscribe to user events with authentication

Returns a stream of UserWsEvent items. The stream will yield events as they are received from the WebSocket connection.

Arguments
  • api_key - API key for authentication
  • api_secret - API secret for authentication
  • api_passphrase - API passphrase for authentication
Events

The stream will yield two types of events:

Errors

Returns an error if:

  • The WebSocket connection fails
  • The authentication message cannot be sent
  • Authentication fails (server will close the connection)